'use strict';

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var A = require('../');
var ServerlessRequestProvider = require('../providers/serverless-request');
var HttpResponseProvider = require('../providers/http-response');
var OutputProvider = require('../providers/output');

module.exports = function () {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

  var providers = [HttpResponseProvider(), ServerlessRequestProvider(), OutputProvider()];

  if (Array.isArray(options.providers)) {
    providers.push.apply(providers, (0, _toConsumableArray3.default)(options.providers));
  }

  var arnie = A({
    providers: providers
  });

  return function (task) {
    return function (event, context, cb) {
      // We could hide the payload and only pass it to provider factories
      var payload = createPayload(event, context);
      return arnie(task, payload).then(function (result) {
        cb(null, createResponse(result));
      }).catch(function (error) {
        // TODO: Find a better way to make errors visible
        // console.error(error)
        cb(error, null);
      });
    };
  };
};

function createPayload(event, context) {
  // TODO: Consider removing functions from event/context
  return {
    serverless: {
      aws: {
        event: event,
        context: context
      }
    }
  };
}

function createResponse(result) {
  var response = {
    statusCode: result.response.statusCode,
    headers: result.response.headers,
    body: result.response.body
  };
  if ((0, _typeof3.default)(response.body) === 'object') {
    try {
      response.body = (0, _stringify2.default)(response.body);
    } catch (error) {
      console.warn('Unable to stringify response body as json: ' + error.message);
      response.body = undefined;
    }
  }
  return response;
}